#!/usr/bin/env python3

import sys
import re
import os

name = os.path.splitext(os.path.splitext(os.path.basename(sys.argv[1]))[0])[0][6:]
var_name = "gsk_gpu_" + name.replace('-', '_')
struct_name = "GskGpu" + name.title().replace('-', '') + "Instance"
n_textures = -1
filename = sys.argv[1]

with open(filename) as f:
    lines = f.readlines()
    matches = []

    for pos, line in enumerate (lines):
        match = re.search(r"^#define GSK_N_TEXTURES ([0-9]+)$", line)
        if match:
            n_textures = int(match.group(1))

        match = re.search(r"^IN\(([0-9]+)\) ([a-z0-9]+) ([a-zA-Z0-9_]+);$", line)
        if not match:
            if re.search(r"layout.*\sin\s.*", line):
                raise Exception(f'''{filename}:{pos}: Failed to parse file''')
            continue
        if not match.group(3).startswith('in'):
            raise Exception(f'''{filename}:{pos}: Variable doesn't start with "in"''')
        matches.append({'name': ''.join('_' + char.lower() if char.isupper() else char for char in match.group(3))[3:],
                        'attrib_name': match.group(3),
                        'location': int(match.group(1)),
                        'type': match.group(2)})

if n_textures < 0:
    raise Exception(f'''{filename}: GSK_N_TEXTURES not defined''')
if n_textures > 2:
    raise Exception(f'''{filename}: GSK_N_TEXTURES must be <= 2''')

print(f'''/* This file is auto-generated; any change will not be preserved */
#pragma once

#define {var_name}_n_textures {n_textures}

typedef struct _{struct_name} {struct_name};

struct _{struct_name} {{''')

expected = 0
for match in matches:
    if expected != int(match['location']):
        raise Exception(f"{filename}: Should be layout location {expected} but is {match['location']}")  # noqa

    if match['type'] == 'float':
        print(f"  float {match['name']};")
        expected += 1
    elif match['type'] == 'int':
        print(f"  gint32 {match['name']};")
        expected += 1
    elif match['type'] == 'uint':
        print(f"  guint32 {match['name']};")
        expected += 1
    elif match['type'] == 'uvec2':
        print(f"  guint32 {match['name']}[2];")
        expected += 1
    elif match['type'] == 'vec2':
        print(f"  float {match['name']}[2];")
        expected += 1
    elif match['type'] == 'vec3':
        print(f"  float {match['name']}[3];")
        expected += 1
    elif match['type'] == 'vec4':
        print(f"  float {match['name']}[4];")
        expected += 1
    elif match['type'] == 'mat3x4':
        print(f"  float {match['name']}[12];")
        expected += 3
    elif match['type'] == 'mat4':
        print(f"  float {match['name']}[16];")
        expected += 4
    else:
        raise Exception(f"{filename}: Don't know what a {match['type']} is")

print(f'''}};
''')

print(f'''static inline void
{var_name}_setup_vao (gsize offset)
{{''')
for i, match in enumerate(matches):
    if match['type'] == 'float':
        print(f'''  glEnableVertexAttribArray ({match['location']});
  glVertexAttribDivisor ({match['location']}, 1);
  glVertexAttribPointer ({match['location']},
                         1,
                         GL_FLOAT,
                         GL_FALSE,
                         sizeof ({struct_name}),
                         GSIZE_TO_POINTER (offset + G_STRUCT_OFFSET({struct_name}, {match['name']})));''');
    elif match['type'] == 'uint':
        print(f'''  glEnableVertexAttribArray ({match['location']});
  glVertexAttribDivisor ({match['location']}, 1);
  glVertexAttribIPointer ({match['location']},
                          1,
                          GL_UNSIGNED_INT,
                          sizeof ({struct_name}),
                          GSIZE_TO_POINTER (offset + G_STRUCT_OFFSET({struct_name}, {match['name']})));''');
    elif match['type'] == 'uvec2':
        print(f'''  glEnableVertexAttribArray ({match['location']});
  glVertexAttribDivisor ({match['location']}, 1);
  glVertexAttribIPointer ({match['location']},
                          2,
                          GL_UNSIGNED_INT,
                          sizeof ({struct_name}),
                          GSIZE_TO_POINTER (offset + G_STRUCT_OFFSET({struct_name}, {match['name']})));''');
    elif match['type'] == 'vec2':
        print(f'''  glEnableVertexAttribArray ({match['location']});
  glVertexAttribDivisor ({match['location']}, 1);
  glVertexAttribPointer ({match['location']},
                         2,
                         GL_FLOAT,
                         GL_FALSE,
                         sizeof ({struct_name}),
                         GSIZE_TO_POINTER (offset + G_STRUCT_OFFSET({struct_name}, {match['name']})));''');
    elif match['type'] == 'vec3':
        print(f'''  glEnableVertexAttribArray ({match['location']});
  glVertexAttribDivisor ({match['location']}, 1);
  glVertexAttribPointer ({match['location']},
                         3,
                         GL_FLOAT,
                         GL_FALSE,
                         sizeof ({struct_name}),
                         GSIZE_TO_POINTER (offset + G_STRUCT_OFFSET({struct_name}, {match['name']})));''');
    elif match['type'] == 'vec4':
        print(f'''  glEnableVertexAttribArray ({match['location']});
  glVertexAttribDivisor ({match['location']}, 1);
  glVertexAttribPointer ({match['location']},
                         4,
                         GL_FLOAT,
                         GL_FALSE,
                         sizeof ({struct_name}),
                         GSIZE_TO_POINTER (offset + G_STRUCT_OFFSET({struct_name}, {match['name']})));''');
    elif match['type'] == 'mat3x4':
        print(f'''  glEnableVertexAttribArray ({match['location']});
  glVertexAttribDivisor ({match['location']}, 1);
  glVertexAttribPointer ({match['location']},
                         4,
                         GL_FLOAT,
                         GL_FALSE,
                         sizeof ({struct_name}),
                         GSIZE_TO_POINTER (offset + G_STRUCT_OFFSET({struct_name}, {match['name']})));
  glEnableVertexAttribArray ({int(match['location'] + 1)});
  glVertexAttribDivisor ({int(match['location'] + 1)}, 1);
  glVertexAttribPointer ({int(match['location']) + 1},
                         4,
                         GL_FLOAT,
                         GL_FALSE,
                         sizeof ({struct_name}),
                         GSIZE_TO_POINTER (offset + G_STRUCT_OFFSET({struct_name}, {match['name']}) + sizeof (float) * 4));
  glEnableVertexAttribArray ({int(match['location'] + 2)});
  glVertexAttribDivisor ({int(match['location'] + 2)}, 1);
  glVertexAttribPointer ({int(match['location']) + 2},
                         4,
                         GL_FLOAT,
                         GL_FALSE,
                         sizeof ({struct_name}),
                         GSIZE_TO_POINTER (offset + G_STRUCT_OFFSET({struct_name}, {match['name']}) + sizeof (float) * 8));''')
    elif match['type'] == 'mat4':
        print(f'''  glEnableVertexAttribArray ({match['location']});
  glVertexAttribDivisor ({match['location']}, 1);
  glVertexAttribPointer ({match['location']},
                         4,
                         GL_FLOAT,
                         GL_FALSE,
                         sizeof ({struct_name}),
                         GSIZE_TO_POINTER (offset + G_STRUCT_OFFSET({struct_name}, {match['name']})));
  glEnableVertexAttribArray ({int(match['location'] + 1)});
  glVertexAttribDivisor ({int(match['location'] + 1)}, 1);
  glVertexAttribPointer ({int(match['location']) + 1},
                         4,
                         GL_FLOAT,
                         GL_FALSE,
                         sizeof ({struct_name}),
                         GSIZE_TO_POINTER (offset + G_STRUCT_OFFSET({struct_name}, {match['name']}) + sizeof (float) * 4));
  glEnableVertexAttribArray ({int(match['location'] + 2)});
  glVertexAttribDivisor ({int(match['location'] + 2)}, 1);
  glVertexAttribPointer ({int(match['location']) + 2},
                         4,
                         GL_FLOAT,
                         GL_FALSE,
                         sizeof ({struct_name}),
                         GSIZE_TO_POINTER (offset + G_STRUCT_OFFSET({struct_name}, {match['name']}) + sizeof (float) * 8));
  glEnableVertexAttribArray ({int(match['location'] + 3)});
  glVertexAttribDivisor ({int(match['location'] + 3)}, 1);
  glVertexAttribPointer ({int(match['location']) + 3},
                         4,
                         GL_FLOAT,
                         GL_FALSE,
                         sizeof ({struct_name}),
                         GSIZE_TO_POINTER (offset + G_STRUCT_OFFSET({struct_name}, {match['name']}) + sizeof (float) * 12));''')
    else:
        raise Exception(f"{filename}: Don't know what a {match['type']} is")

print(f'''}}

''');


print(f'''static void
{var_name}_setup_attrib_locations (GLuint program)
{{''')

for match in matches:
    print(f'''  glBindAttribLocation (program, {match['location']}, "{match['attrib_name']}");''')

print(f'''}}

''');


print(f'''#ifdef GDK_RENDERING_VULKAN

static const VkPipelineVertexInputStateCreateInfo {var_name}_info = {{
  .sType = VK_STRUCTURE_TYPE_PIPELINE_VERTEX_INPUT_STATE_CREATE_INFO,
  .vertexBindingDescriptionCount = 1,
  .pVertexBindingDescriptions = (VkVertexInputBindingDescription[1]) {{
      {{
        .binding = 0,
        .stride = sizeof ({struct_name}),
        .inputRate = VK_VERTEX_INPUT_RATE_INSTANCE
      }}
  }},
  .vertexAttributeDescriptionCount = {expected},
  .pVertexAttributeDescriptions = (VkVertexInputAttributeDescription[{expected}]) {{''')

for match in matches:
    if match['type'] == 'float':
        print(f'''      {{
        .location = {match['location']},
        .binding = 0,
        .format = VK_FORMAT_R32_SFLOAT,
        .offset = G_STRUCT_OFFSET({struct_name}, {match['name']}),
      }},''')
    elif match['type'] == 'int':
        print(f'''      {{
        .location = {match['location']},
        .binding = 0,
        .format = VK_FORMAT_R32_SINT,
        .offset = G_STRUCT_OFFSET({struct_name}, {match['name']}),
      }},''')

    elif match['type'] == 'uint':
        print(f'''      {{
        .location = {match['location']},
        .binding = 0,
        .format = VK_FORMAT_R32_UINT,
        .offset = G_STRUCT_OFFSET({struct_name}, {match['name']}),
      }},''')
    elif match['type'] == 'uvec2':
        print(f'''      {{
        .location = {match['location']},
        .binding = 0,
        .format = VK_FORMAT_R32G32_UINT,
        .offset = G_STRUCT_OFFSET({struct_name}, {match['name']}),
      }},''')
    elif match['type'] == 'vec2':
        print(f'''      {{
        .location = {match['location']},
        .binding = 0,
        .format = VK_FORMAT_R32G32_SFLOAT,
        .offset = G_STRUCT_OFFSET({struct_name}, {match['name']}),
      }},''')
    elif match['type'] == 'vec3':
        print(f'''      {{
        .location = {match['location']},
        .binding = 0,
        .format = VK_FORMAT_R32G32B32_SFLOAT,
        .offset = G_STRUCT_OFFSET({struct_name}, {match['name']}),
      }},''')
    elif match['type'] == 'vec4':
        print(f'''      {{
        .location = {match['location']},
        .binding = 0,
        .format = VK_FORMAT_R32G32B32A32_SFLOAT,
        .offset = G_STRUCT_OFFSET({struct_name}, {match['name']}),
      }},''')
    elif match['type'] == 'mat3x4':
        print(f'''      {{
        .location = {match['location']},
        .binding = 0,
        .format = VK_FORMAT_R32G32B32A32_SFLOAT,
        .offset = G_STRUCT_OFFSET({struct_name}, {match['name']}),
      }},
      {{
        .location = {int(match['location']) + 1},
        .binding = 0,
        .format = VK_FORMAT_R32G32B32A32_SFLOAT,
        .offset = G_STRUCT_OFFSET({struct_name}, {match['name']}) + sizeof (float) * 4,
      }},
      {{
        .location = {int(match['location']) + 2},
        .binding = 0,
        .format = VK_FORMAT_R32G32B32A32_SFLOAT,
        .offset = G_STRUCT_OFFSET({struct_name}, {match['name']}) + sizeof (float) * 8,
      }},''')
    elif match['type'] == 'mat4':
        print(f'''      {{
        .location = {match['location']},
        .binding = 0,
        .format = VK_FORMAT_R32G32B32A32_SFLOAT,
        .offset = G_STRUCT_OFFSET({struct_name}, {match['name']}),
      }},
      {{
        .location = {int(match['location']) + 1},
        .binding = 0,
        .format = VK_FORMAT_R32G32B32A32_SFLOAT,
        .offset = G_STRUCT_OFFSET({struct_name}, {match['name']}) + sizeof (float) * 4,
      }},
      {{
        .location = {int(match['location']) + 2},
        .binding = 0,
        .format = VK_FORMAT_R32G32B32A32_SFLOAT,
        .offset = G_STRUCT_OFFSET({struct_name}, {match['name']}) + sizeof (float) * 8,
      }},
      {{
        .location = {int(match['location']) + 3},
        .binding = 0,
        .format = VK_FORMAT_R32G32B32A32_SFLOAT,
        .offset = G_STRUCT_OFFSET({struct_name}, {match['name']}) + sizeof (float) * 12,
      }},''')
    else:
        raise Exception(f"{filename}: Don't know what a {match['type']} is")

print(f'''  }},
}};

#endif
''');



#!/usr/bin/env python3

# Turns a Mason testlog.json file into an HTML report
#
# Copyright 2019  GNOME Foundation
#
# SPDX-License-Identifier: LGPL-2.1-or-later
#
# Original author: Emmanuele Bassi

import argparse
import datetime
import json
import os
import sys
from jinja2 import Template

REPORT_TEMPLATE = '''
<!DOCTYPE html>
<html lang="en">
<head>
  <title>{{ report.project_name }} Test Report</title>
  <meta charset="utf-8" />
  <style type="text/css">
body {
  background: white;
  color: #333;
  font-family: 'Cantarell', sans-serif;
}

h1 {
  color: #333333;
  font-size: 1.9em;
  font-weight: normal;
  margin-bottom: 1em;
  border-bottom: 1px solid #333333;
}

header {
  position: fixed;
  padding-bottom: 12px;
  margin-bottom: 24px;
  background: rgba(255, 255, 255, 0.85);
  box-shadow: 0 0 1px rgba(0, 0, 0, 0.15);
  z-index: 500;
  left: 0;
  top: 0;
  width: 100%;
  color: rgba(0, 0, 0, 0.3);
  transform: translateY(0px);
  transition: .2s background-color, color;
  box-sizing: border-box;
  display: block;
  visibility: visible;
  text-align: center;
}

article {
  padding-top: 200px;
  margin: 2em;
}

div.report-meta {
  width: auto;
  border: 1px solid #ccc;
  padding: .5em 2em;
  color: #3c3c3c;
}

span.result {
  font-weight: bold;
}

span.pass {
  color: rgb(51, 209, 122);
}

span.skip {
  color: rgb(255, 163, 72);
}

span.fail {
  color: rgb(224, 27, 36);
}

span.xfail {
  color: rgb(163, 71, 186);
}

div.result {
  border-top: 1px solid #c0c0c0;
  padding-top: 1em;
  padding-bottom: 1em;
  width: 100%;
}

div.result h4 {
  border-bottom: 1px solid #c0c0c0;
  margin-bottom: 0.7em;
}

pre {
  color: #fafafa;
  background-color: black;
  border-radius: 6px;
  box-shadow: 0px 5px 8px 0px rgba(0, 0, 0, 0.25);
  font-family: monospace;
  line-height: 1.2em;
  border: none;
  padding: 10px 1em;
  font-size: 0.9em;
  overflow: auto;
  white-space: pre;
  word-break: normal;
  word-wrap: normal;
}

ul.passed li {
  display: inline;
}

ul.passed li:after {
  content: ",";
}

ul.passed li:last-child:after {
  content: "";
}

ul.images {
  padding-bottom: 1em;
}

ul.images li {
  display: inline;
}
  </style>
</head>
<body>
  <header>
    <h1>{{ report.project_name }}/{{ report.backend }}/{{ report.branch_name }} :: Test Reports</h1>
  </header>

  <article>
    <section>
      <div class="report-meta">
        <p><strong>Backend:</strong> {{ report.backend }}</p>
        <p><strong>Branch:</strong> {{ report.branch_name }}</p>
        <p><strong>Date:</strong> <time datetime="{{ report.date.isoformat() }}">{{ report.locale_date }}</time></p>
        {% if report.job_id %}<p><strong>Job ID:</strong> {{ report.job_id }}</p>{% endif %}
      </div>
    </section>

    <section>
      <div class="summary">
        <h3><a name="summary">Summary</a></h3>
        <ul>
          <li><strong>Total units:</strong> {{ report.total_units }}</li>
          <li><strong>Failed:</strong> {{ report.total_failures }}</li>
          <li><strong>Passed:</strong> {{ report.total_successes }}</li>
        </ul>
      </div>
    </section>

    {% for suite_result in report.results_list %}
    <section>
      <div class="result">
        <h3><a name="results">Suite: {{ suite_result.suite_name }}</a></h3>
        <ul>
          <li><strong>Units:</strong> {{ suite_result.n_units }}</li>
          <li><strong>Failed:</strong> <a href="#{{ suite_result.suite_name }}-failed">{{ suite_result.n_failures }}</a></li>
          <li><strong>Passed:</strong> <a href="#{{ suite_result.suite_name }}-passed">{{ suite_result.n_successes }}</a></li>
        </ul>

        <div class="failures">
          <h4><a name="{{ suite_result.suite_name }}-failed">Failures</a></h4>
          <ul class="failed">
            {% for failure in suite_result.failures if failure.result in [ 'ERROR', 'FAIL', 'UNEXPECTEDPASS' ] %}
            <li><a name="{{ failure.name }}">{{ failure.name }}</a> - result: <span class="result fail">{{ failure.result }}</span><br/>
              {% if failure.stdout %}
              Output: <pre>{{ failure.stdout }}</pre>
              {% endif %}
              {% if failure.image_data is defined %}
              <ul class="images">
                <li><img alt="ref" src="{{ failure.image_data.ref }}" /></li>
                <li><img alt="out" src="{{ failure.image_data.out }}" /></li>
                <li><img alt="diff" src="{{ failure.image_data.diff }}" /></li>
                <li><a href="{{ failure.image_data.refnode }}">ref node</a></li>
                <li><a href="{{ failure.image_data.outnode }}">out node</a></li>
              </ul>
              {% endif %}
            </li>
            {% else %}
            <li>None</li>
            {% endfor %}
          </ul>

          <h4><a name="{{ suite_result.suite_name }}-timed-out">Timed out</a></h4>
          <ul class="failed">
            {% for failure in suite_result.failures if failure.result == 'TIMEOUT' %}
            <li><a name="{{ failure.name }}">{{ failure.name }}</a> - result: <span class="result fail">{{ failure.result }}</span><br/>
              {% if failure.stdout %}
              Output: <pre>{{ failure.stdout }}</pre>
              {% endif %}
            </li>
            {% else %}
            <li>None</li>
            {% endfor %}
          </ul>
        </div>

        <div class="successes">
          <h4><a name="{{ suite_result.suite_name }}-expected-fail">Expected failures</a></h4>
          <ul>
          {% for success in suite_result.successes if success.result == 'EXPECTEDFAIL' %}
            <li><a name="{{ success.name }}">{{ success.name }}</a> - result: <span class="result xfail">{{ success.result }}</span><br/>
            {% if success.stdout %}
              Output: <pre>{{ success.stdout }}</pre>
            {% endif %}
            {% if success.image_data is defined %}
              <ul class="images">
                <li><img alt="ref" src="{{ success.image_data.ref }}" /></li>
                <li><img alt="out" src="{{ success.image_data.out }}" /></li>
                <li><img alt="diff" src="{{ success.image_data.diff }}" /></li>
              </ul>
            {% endif %}
            </li>
          {% else %}
            <li>None</li>
          {% endfor %}
          </ul>

          <h4><a name="{{ suite_result.suite_name }}-skipped">Skipped</a></h4>
          <ul>
            {% for success in suite_result.successes if success.result == 'SKIP' %}
            <li>{{ success.name }} - result: <span class="result skip">{{ success.result }}</li>
            {% else %}
            <li>None</li>
            {% endfor %}
          </ul>

          <h4><a name="{{ suite_result.suite_name }}-passed">Passed</a></h4>
          <ul class="passed">
            {% for success in suite_result.successes if success.result == 'OK' %}
            <li>{{ success.name }} - result: <span class="result pass">{{ success.result }}</li>
            {% else %}
            <li>None</li>
            {% endfor %}
          </ul>
        </div>

      </div>
    </section>
    {% endfor %}

  </article>
</body>
</html>
'''

aparser = argparse.ArgumentParser(description='Turns a Meson test log into an HTML report')
aparser.add_argument('--project-name', metavar='NAME',
                     help='The project name',
                     default='Unknown')
aparser.add_argument('--backend', metavar='NAME',
                     help='The used backend',
                     default='unknown')
aparser.add_argument('--job-id', metavar='ID',
                     help='The job ID for the report',
                     default=None)
aparser.add_argument('--branch', metavar='NAME',
                     help='Branch of the project being tested',
                     default='main')
aparser.add_argument('--output', metavar='FILE',
                     help='The output HTML file, stdout by default',
                     type=argparse.FileType('w', encoding='UTF-8'),
                     default=sys.stdout)
aparser.add_argument('--reftest-suite', metavar='NAME',
                     help='The name of the reftests suite',
                     default='reftest')
aparser.add_argument('--reftest-output-dir', metavar='DIR',
                     help='The output directory for reftests data',
                     default=None)
aparser.add_argument('infile', metavar='FILE',
                     help='The input testlog.json, stdin by default',
                     type=argparse.FileType('r', encoding='UTF-8'),
                     default=sys.stdin)

args = aparser.parse_args()

outfile = args.output

suites = {}
for line in args.infile:
    data = json.loads(line)
    (full_suite, unit_name) = data['name'].split(' / ')
    (project_name, suite_name) = full_suite.split(':')

    unit = {
        'project-name': project_name,
        'suite': suite_name,
        'name': unit_name,
        'duration': data['duration'],
        'returncode': data['returncode'],
        'result': data['result'],
        'stdout': data['stdout'],
    }

    if args.reftest_output_dir is not None and suite_name == args.reftest_suite:
        filename = unit_name.split(' ')[1]
        basename = os.path.splitext(filename)[0]

        image_data = {
            'ref': os.path.join(args.reftest_output_dir, '{}.ref.png'.format(basename)),
            'out': os.path.join(args.reftest_output_dir, '{}.out.png'.format(basename)),
            'refnode': os.path.join(args.reftest_output_dir, '{}.ref.node'.format(basename)),
            'outnode': os.path.join(args.reftest_output_dir, '{}.out.node'.format(basename)),
            'diff': os.path.join(args.reftest_output_dir, '{}.diff.png'.format(basename)),
        }

        unit['image_data'] = image_data

    units = suites.setdefault(full_suite, [])
    units.append(unit)

report = {}
report['date'] = datetime.datetime.now(datetime.UTC)
report['locale_date'] = report['date'].strftime("%c")
report['project_name'] = args.project_name
report['backend'] = args.backend
report['job_id'] = args.job_id
report['branch_name'] = args.branch
report['total_successes'] = 0
report['total_failures'] = 0
report['total_units'] = 0
report['results_list'] = []

for name, units in suites.items():
    (project_name, suite_name) = name.split(':')
    print('Processing {} suite {}:'.format(project_name, suite_name))

    def if_failed(unit):
        if unit['result'] in ['FAIL', 'UNEXPECTEDPASS', 'TIMEOUT', 'ERROR',]:
            return True
        return False

    def if_succeded(unit):
        if unit['result'] in ['OK', 'EXPECTEDFAIL', 'SKIP']:
            return True
        return False

    successes = list(filter(if_succeded, units))
    failures = list(filter(if_failed, units))

    n_units = len(units)
    n_successes = len(successes)
    n_failures = len(failures)

    report['total_units'] += n_units
    report['total_successes'] += n_successes
    report['total_failures'] += n_failures
    print(' - {}: {} total, {} pass, {} fail'.format(suite_name, n_units, n_successes, n_failures))

    suite_report = {
        'suite_name': suite_name,
        'n_units': n_units,
        'successes': successes,
        'n_successes': n_successes,
        'failures': failures,
        'n_failures': n_failures,
    }
    report['results_list'].append(suite_report)

template = Template(REPORT_TEMPLATE)
outfile.write(template.render({'report': report}))



#!/usr/bin/env python3

import sys
import re
import os

loaded_files = []

check_defines = [ 'VULKAN' ]

def load (path):
    if (path in loaded_files):
        return

    loaded_files.append (path)
    skipping = ''

    with open(path) as f:
        lines = f.readlines()
        for line in lines:
            if skipping:
                match = re.search (r"^#endif /\* (.*) \*/$", line)
                if match and match.group(1) == skipping:
                    skipping = ''
                continue

            match = re.search (r"^#define (.*)$", line)
            if match and match.group(1) in check_defines:
                check_defines.remove (match.group(1))
                print (line, end="")
                continue

            match = re.search (r"^#ifdef (.*)$", line)
            if match and match.group(1) in check_defines:
                skipping = match.group(1)
                continue

            match = re.search (r"^#include \"(.*)\"$", line)
            if match:
                load (os.path.join (os.path.dirname(path), match.group(1)))
            else:
                print (line, end="")

load (sys.argv[1])




#!/usr/bin/env python3
#
# Generate gsk.resources.xml
#
# Usage: gen-gsk-gresources-xml OUTPUT-FILE [INPUT-FILE1] [INPUT-FILE2] ...

import os, sys
import filecmp

def replace_if_changed(new, old):
  '''
  Compare contents and only replace if changed to avoid triggering a rebuild.
  '''
  try:
    changed = not filecmp.cmp(new, old, shallow=False)
  except FileNotFoundError:
    changed = True
  if changed:
    os.replace(new, old)
  else:
    os.remove(new)

gl_source_shaders = []
ngl_source_shaders = []
vulkan_compiled_shaders = []
gpu_vulkan_compiled_shaders = []
vulkan_shaders = []

for f in sys.argv[2:]:
  if f.endswith('.glsl'):
    if f.find('gsk/gpu') > -1:
      ngl_source_shaders.append(f)
    else:
      gl_source_shaders.append(f)
  elif f.endswith('.spv'):
    if f.find('gsk/gpu') > -1:
      gpu_vulkan_compiled_shaders.append(f)
    else:
      vulkan_compiled_shaders.append(f)
  elif f.endswith('.frag') or f.endswith('.vert'):
    vulkan_shaders.append(f)
  else:
    raise Exception(f"No idea what XML to generate for {f}")

xml = '''<?xml version='1.0' encoding='UTF-8'?>
<gresources>
  <gresource prefix='/org/gtk/libgsk'>

'''

for f in gl_source_shaders:
  xml += '    <file alias=\'gl/{0}\'>gl/resources/{0}</file>\n'.format(os.path.basename(f))

xml += '\n'

for f in ngl_source_shaders:
  xml += '    <file alias=\'shaders/gl/{0}\'>gpu/shaders/{0}</file>\n'.format(os.path.basename(f))

xml += '\n'

for f in vulkan_compiled_shaders:
  xml += '    <file alias=\'vulkan/{0}\'>vulkan/resources/{0}</file>\n'.format(os.path.basename(f))

xml += '\n'

for f in gpu_vulkan_compiled_shaders:
  xml += '    <file alias=\'shaders/vulkan/{0}\'>gpu/shaders/{0}</file>\n'.format(os.path.basename(f))

xml += '\n'

for f in vulkan_shaders:
  xml += '    <file alias=\'vulkan/{0}\'>vulkan/resources/{0}</file>\n'.format(os.path.basename(f))

xml += '''
  </gresource>
</gresources>'''

if len(sys.argv) > 1 and sys.argv[1] != '-':
  outfile = sys.argv[1]
  tmpfile = outfile + '~'
  with open(tmpfile, 'w') as f:
    f.write(xml)
  replace_if_changed(tmpfile, outfile)
else:
  print(xml)



import sys
import os
import subprocess
import gi

gi.require_version('Gdk', '4.0')
gi.require_version('Gtk', '4.0')

from gi.repository import GLib, GObject, Gdk, Gtk
from pydbus import SessionBus

verbose = True

remote_desktop = None
screen_cast = None
session = None
stream_path = None
done = False

def terminate():
    sys.exit(1)

loop = None

def quit_cb(loop):
    loop.quit()
    print('timed out while waiting')

def wait(millis):
    global loop
    before = GLib.get_monotonic_time()
    loop = GLib.MainLoop()
    GLib.timeout_add(millis, quit_cb, loop)
    loop.run()
    if verbose:
        time = (GLib.get_monotonic_time() - before) / 1000
        print(f'waited for {time} milliseconds')

display = None
window = None
entry = None
expected_change = None

def key_pressed_cb (controller, keyval, keycode, state):
    global expected_change
    global loop

    if verbose:
        print(f'got key press: {keyval}, state {state}')
    assert expected_change != None, 'Unexpected key press'
    assert expected_change['type'] == 'press', 'Key press event expected'
    assert keyval == expected_change['keyval'], 'Unexpected keyval in key press event'
    assert state == expected_change['state'], 'Unexpected state in key press event'

    expected_change = None
    loop.quit()

def key_released_cb (controller, keyval, keycode, state):
    global expected_change
    global loop

    if verbose:
        print(f'got key release: {keyval}, state {state}')
    assert expected_change != None, 'Unexpected key release'
    assert expected_change['type'] == 'release', 'Key release event expected'
    assert keyval == expected_change['keyval'], 'Unexpected keyval in key release event'
    assert state == expected_change['state'], 'Unexpected state in key release event'

    expected_change = None
    loop.quit()

def motion_cb (controller, x, y):
    global expected_change
    global loop

    if verbose:
        print(f'got motion: {x}, {y}')
    if expected_change != None:
        assert expected_change['type'] == 'motion', 'Motion event expected'
        assert x == expected_change['x'], 'Unexpected x coord in motion event'
        assert y == expected_change['y'], 'Unexpected y coord in motion event'
        expected_change = None
    loop.quit()

def enter_cb (controller, x, y):
    global expected_change
    global loop

    if verbose:
        print(f'got enter: {x}, {y}')
    assert expected_change != None, 'Unexpected enter'
    assert expected_change['type'] == 'enter', 'Enter event expected'
    assert x == expected_change['x'], 'Unexpected x coord in enter event'
    assert y == expected_change['y'], 'Unexpected y coord in enter event'

    expected_change = None
    loop.quit()

def pressed_cb(controller, n, x, y):
    global expected_change
    global loop

    if verbose:
        print(f'got pressed')
    assert expected_change != None, 'Unexpected event'
    assert expected_change['type'] == 'press', 'Button press expected'
    assert expected_change['button'] == controller.get_current_button(), 'Unexpected button pressed'
    assert x == expected_change['x'], 'Unexpected x coord in motion event'
    assert y == expected_change['y'], 'Unexpected y coord in motion event'

    expected_change = None
    loop.quit()

def released_cb(controller, n, x, y):
    global expected_change
    global loop

    if verbose:
        print(f'got released')
    assert expected_change != None, 'Unexpected event'
    assert expected_change['type'] == 'release', 'Button release expected'

    expected_change = None
    loop.quit()

def expect_key_press(keyval, state, timeout):
    global expected_change
    expected_change = {
      'type' : 'press',
      'keyval' : keyval,
      'state' : state
    }
    wait(timeout)
    assert expected_change == None, 'Expected event did not happen'

def expect_key_release(keyval, state, timeout):
    global expected_change
    expected_change = {
      'type' : 'release',
      'keyval' : keyval,
      'state' : state
    }
    wait(timeout)
    assert expected_change == None, 'Expected event did not happen'

def expect_motion(x, y, timeout):
    global expected_change
    expected_change = {
      'type' : 'motion',
      'x' : x,
      'y' : y
    }
    wait(timeout)
    assert expected_change == None, 'Expected event did not happen'

def expect_enter(x, y, timeout):
    global expected_change
    expected_change = {
      'type' : 'enter',
      'x' : x,
      'y' : y
    }
    wait(timeout)
    assert expected_change == None, 'Expected event did not happen'

def expect_button_press(button, x, y, timeout):
    global expected_change
    expected_change = {
      'type' : 'press',
      'button' : button,
      'x' : x,
      'y' : y
    }
    wait(timeout)
    assert expected_change == None, 'Button press did not arrive'

def expect_button_release(button, x, y, timeout):
    global expected_change
    expected_change = {
      'type' : 'release',
      'button' : button,
      'x' : x,
      'y' : y
    }
    wait(timeout)
    assert expected_change == None, 'Button release did not arrive'

def got_active(object, pspec):
    global loop
    object.disconnect_by_func(got_active)
    loop.quit()

def launch_observer():
    global display
    global window

    if verbose:
        print('launch observer')

    if display == None:
        display = Gdk.Display.open(os.getenv('WAYLAND_DISPLAY'))

    window = Gtk.Window.new()

    controller = Gtk.EventControllerKey.new()
    controller.set_propagation_phase(Gtk.PropagationPhase.CAPTURE)
    controller.connect('key-pressed', key_pressed_cb)
    controller.connect('key-released', key_released_cb)
    window.add_controller(controller)

    controller = Gtk.EventControllerMotion.new()
    controller.set_propagation_phase(Gtk.PropagationPhase.CAPTURE)
    controller.connect('enter', enter_cb)
    controller.connect('motion', motion_cb)
    window.add_controller(controller)

    controller = Gtk.GestureClick.new()
    controller.set_propagation_phase(Gtk.PropagationPhase.CAPTURE)
    controller.connect('pressed', pressed_cb)
    controller.connect('released', released_cb)
    window.add_controller(controller)

    window.connect('notify::is-active', got_active)
    window.maximize()
    window.present()

    wait(2000)

    assert window.is_active(), 'Observer not active'
    assert window.get_width() == 1024, 'Window not maximized'
    assert window.get_height() == 768, 'Window not maximized'

    # we need to wait out the map animation, or pointer coords will be off
    wait(1000)

def launch_entry():
    global display
    global window
    global entry

    if verbose:
        print('launch entry')

    if display == None:
        display = Gdk.Display.open(os.getenv('WAYLAND_DISPLAY'))

    window = Gtk.Window.new()

    entry = Gtk.Entry.new()

    window.set_child(entry)

    window.connect('notify::is-active', got_active)
    window.maximize()
    window.present()

    wait(500)

    assert window.is_active(), "Observer not active"
    assert window.get_width() == 1024, "Window not maximized"
    assert window.get_height() == 768, "Window not maximized"

    # we need to wait out the map animation, or pointer coords will be off
    wait(1000)

def stop_observer():
    global window
    window.destroy()
    window = None

def expect_entry_text(text):
    assert text == entry.get_text(), 'Unexpected entry text: ' + entry.get_text()

def key_press(keyval):
    if verbose:
        print(f'press key {keyval}')
    session.NotifyKeyboardKeysym(keyval, True)

def key_release(keyval):
    if verbose:
        print(f'release key {keyval}')
    session.NotifyKeyboardKeysym(keyval, False)

buttons = {
    1 : 0x110,
    2 : 0x111,
    3 : 0x112
}

def button_press(button):
    if verbose:
        print(f'press button {button}')
    session.NotifyPointerButton(buttons[button], True)

def button_release(button):
    if verbose:
        print(f'release button {button}')
    session.NotifyPointerButton(buttons[button], False)

def pointer_move(x, y):
    if verbose:
        print(f'pointer move {x} {y}')
    session.NotifyPointerMotionAbsolute(stream_path, x, y)

def basic_keyboard_tests():
    try:
        if verbose:
            print('Starting basic keyboard tests')

        launch_observer()

        key_press(Gdk.KEY_a)
        expect_key_press(keyval=Gdk.KEY_a, state=0, timeout=100)

        key_release(Gdk.KEY_a)
        expect_key_release(keyval=Gdk.KEY_a, state=0, timeout=100)

        key_press(Gdk.KEY_Control_L)
        expect_key_press(keyval=Gdk.KEY_Control_L, state=0, timeout=100)

        key_press(Gdk.KEY_x)
        expect_key_press(keyval=Gdk.KEY_x, state=Gdk.ModifierType.CONTROL_MASK, timeout=100)

        key_release(Gdk.KEY_Control_L)
        expect_key_release(keyval=Gdk.KEY_Control_L, state=Gdk.ModifierType.CONTROL_MASK, timeout=100)

        key_release(Gdk.KEY_x)
        expect_key_release(keyval=Gdk.KEY_x, state=0, timeout=100)

        stop_observer()
    except AssertionError as e:
        print(f'Error in basic_keyboard_tests: {e}')
        terminate()

def quick_typing_test():
    try:
        launch_entry()

        key_press(Gdk.KEY_T)
        key_release(Gdk.KEY_T)
        key_press(Gdk.KEY_e)
        key_release(Gdk.KEY_e)
        key_press(Gdk.KEY_s)
        key_release(Gdk.KEY_s)
        key_press(Gdk.KEY_t)
        key_release(Gdk.KEY_t)

        wait(100)
        expect_entry_text('Test')

        stop_observer()
    except AssertionError as e:
        print(f'Error in quick_typing_test: {e}')
        terminate()

def basic_pointer_tests():
    try:
        if verbose:
            print('Starting basic pointer tests')

        pointer_move(-100.0, -100.0)
        launch_observer()

        # observer window is maximized, so window coords == global coords
        pointer_move(500.0, 300.0)
        expect_enter(x=500, y=300, timeout=200)

        pointer_move(400.0, 200.0)
        expect_motion(x=400, y=200, timeout=200)

        button_press(1)
        expect_button_press(button=1, x=400, y=200, timeout=200)

        pointer_move(220.0, 200.0)
        expect_motion(x=220, y=200, timeout=200)

        button_release(1)
        expect_button_release(button=1, x=220, y=200, timeout=200)

        stop_observer()
    except AssertionError as e:
        print(f'Error in basic_pointer_tests: {e}')
        terminate()

ds_window = None
ds = None

def drag_begin(controller, drag):
    global expected_change
    global loop

    if verbose:
        print('got drag begin')
    assert expected_change != None, 'Unexpected drag begin'
    assert expected_change['type'] == 'drag', 'Drag begin expected'

    expected_change = None
    loop.quit()

def launch_drag_source(value):
    global display
    global ds_window
    global ds

    if verbose:
        print('launch drag source')

    if display == None:
        display = Gdk.Display.open(os.getenv('WAYLAND_DISPLAY'))

    ds_window = Gtk.Window.new()
    ds_window.set_title('Drag Source')

    ds = Gtk.DragSource.new()
    ds.set_content(Gdk.ContentProvider.new_for_value(value))
    ds_window.add_controller(ds)
    ds.connect('drag-begin', drag_begin)

    controller = Gtk.GestureClick.new()
    controller.set_propagation_phase(Gtk.PropagationPhase.CAPTURE)
    controller.connect('pressed', pressed_cb)
    controller.connect('released', released_cb)
    ds_window.add_controller(controller)

    ds_window.connect('notify::is-active', got_active)
    ds_window.maximize()
    ds_window.present()

    wait(2000)

    assert ds_window.is_active(), 'drag source not active'
    assert ds_window.get_width() == 1024, 'Window not maximized'
    assert ds_window.get_height() == 768, 'Window not maximized'

    # we need to wait out the map animation, or pointer coords will be off
    wait(1000)

def stop_drag_source():
    global ds_window
    ds_window.destroy()
    ds_window = None

dt_window = None

def do_drop(controller, value, x, y):
    global expected_change
    global loop

    if verbose:
        print(f'got drop {value}')
    assert expected_change != None, 'Unexpected drop begin'
    assert expected_change['type'] == 'drop', 'Drop expected'
    assert expected_change['value'] == value, 'Unexpected value dropped'

    expected_change = None
    loop.quit()

def launch_drop_target():
    global display
    global dt_window

    if verbose:
        print('launch drop target')

    if display == None:
        display = Gdk.Display.open(os.getenv('WAYLAND_DISPLAY'))

    dt_window = Gtk.Window.new()
    dt_window.set_title('Drop Target')

    controller = Gtk.DropTarget.new(GObject.TYPE_STRING, Gdk.DragAction.COPY)
    dt_window.add_controller(controller)
    controller.connect('drop', do_drop)

    dt_window.connect('notify::is-active', got_active)
    dt_window.maximize()
    dt_window.present()

    wait(2000)

    assert dt_window.is_active(), 'drop target not active'
    assert dt_window.get_width() == 1024, 'Window not maximized'
    assert dt_window.get_height() == 768, 'Window not maximized'

    # we need to wait out the map animation, or pointer coords will be off
    wait(1000)

def stop_drop_target():
    global dt_window
    dt_window.destroy()
    dt_window = None

def expect_drag(timeout):
    global expected_change
    expected_change = {
      'type' : 'drag',
    }
    wait(timeout)
    assert expected_change == None, 'DND operation not started'

def expect_drop(value, timeout):
    global expected_change
    expected_change = {
      'type' : 'drop',
      'value' : value
    }
    wait(timeout)
    assert expected_change == None, 'Drop has not happened'

def dnd_tests():
    try:
        if verbose:
            print('Starting dnd tests')

        pointer_move(-100, -100)

        launch_drag_source('abc')
        wait(100);

        pointer_move(100, 100)
        wait(100);
        button_press(1)
        expect_button_press(button=1, x=100, y=100, timeout=300)
        # need to wait out the MIN_TIME_TO_DND
        wait(150)

        pointer_move(120, 150)
        expect_drag(timeout=2000)

        launch_drop_target()
        wait(100);
        button_release(1)
        expect_drop('abc', timeout=2000)

        stop_drop_target()
        stop_drag_source()
    except AssertionError as e:
        print(f'Error in dnd_tests: {e}')
        terminate()

def session_closed_cb():
    print('Session closed')

def run_commands():
    basic_keyboard_tests()
    basic_pointer_tests()
    dnd_tests()
    quick_typing_test()

def mutter_appeared(name):
    global remote_desktop
    global session
    global stream_path
    global done

    if verbose:
        print('mutter appeared on the bus')

    remote_desktop = bus.get('org.gnome.Mutter.RemoteDesktop',
                             '/org/gnome/Mutter/RemoteDesktop')
    device_types = remote_desktop.Get('org.gnome.Mutter.RemoteDesktop', 'SupportedDeviceTypes')
    assert device_types & 1 == 1, 'No keyboard'
    assert device_types & 2 == 2, 'No pointer'

    screen_cast = bus.get('org.gnome.Mutter.ScreenCast',
                          '/org/gnome/Mutter/ScreenCast')

    session_path = remote_desktop.CreateSession()
    session = bus.get('org.gnome.Mutter.RemoteDesktop', session_path)
    session.onClosed = session_closed_cb

    screen_cast_session_path = screen_cast.CreateSession({ 'remote-desktop-session-id' : GLib.Variant('s', session.SessionId)})
    screen_cast_session = bus.get('org.gnome.Mutter.ScreenCast', screen_cast_session_path)

    stream_path = screen_cast_session.RecordMonitor('Meta-0', {})
    session.Start()

    # work around lack of initial devices
    key_press(Gdk.KEY_Control_L)
    key_release(Gdk.KEY_Control_L)
    pointer_move(-100, -100)

    run_commands()

    session.Stop()

    if verbose:
        print('Done running commands, exiting...')
    done = True

def mutter_vanished():
    global done
    if remote_desktop != None:
        if verbose:
            print('mutter left the bus')
        done = True

bus = SessionBus()
bus.watch_name('org.gnome.Mutter.RemoteDesktop', 0, mutter_appeared, mutter_vanished)

try:
    while not done:
      GLib.MainContext.default().iteration(True)
except KeyboardInterrupt:
    print('Interrupted')



#!/usr/bin/env python3
#
# Generate gtk.gresources.xml
#
# Usage: gen-gtk-gresources-xml SRCDIR_GTK [OUTPUT-FILE]

import os, sys
import filecmp

def replace_if_changed(new, old):
  '''
  Compare contents and only replace if changed to avoid triggering a rebuild.
  '''
  try:
    changed = not filecmp.cmp(new, old, shallow=False)
  except FileNotFoundError:
    changed = True
  if changed:
    os.replace(new, old)
  else:
    os.remove(new)

srcdir = sys.argv[1]
endian = sys.argv[2]

xml = '''<?xml version='1.0' encoding='UTF-8'?>
<gresources>
  <gresource prefix='/org/gtk/libgtk'>
'''

def get_files(subdir,extension):
  return sorted(filter(lambda x: x.endswith((extension)), os.listdir(os.path.join(srcdir,subdir))))

xml += '''
    <file>theme/Empty/gtk.css</file>

    <file>theme/Default/gtk.css</file>
    <file alias='theme/Default-dark/gtk.css'>theme/Default/gtk-dark.css</file>
    <file alias='theme/Default-hc/gtk.css'>theme/Default/gtk-hc.css</file>
    <file alias='theme/Default-hc-dark/gtk.css'>theme/Default/gtk-hc-dark.css</file>

    <file>theme/Default/gtk-light.css</file>
    <file>theme/Default/gtk-dark.css</file>
    <file>theme/Default/gtk-hc.css</file>
    <file>theme/Default/gtk-hc-dark.css</file>
    <file>theme/Default/Default-light.css</file>
    <file>theme/Default/Default-dark.css</file>
    <file>theme/Default/Default-hc.css</file>
    <file>theme/Default/Default-hc-dark.css</file>
'''

for f in get_files('theme/Default/assets', '.png'):
  xml += '    <file>theme/Default/assets/{0}</file>\n'.format(f)

xml += '\n'

for f in get_files('theme/Default/assets', '.svg'):
  xml += '    <file preprocess=\'xml-stripblanks\'>theme/Default/assets/{0}</file>\n'.format(f)

for f in get_files('theme/Default/assets-hc', '.png'):
  xml += '    <file>theme/Default/assets-hc/{0}</file>\n'.format(f)

xml += '\n'

for f in get_files('theme/Default/assets-hc', '.svg'):
  xml += '    <file preprocess=\'xml-stripblanks\'>theme/Default/assets-hc/{0}</file>\n'.format(f)

for f in get_files('ui', '.ui'):
  xml += '    <file>ui/{0}</file>\n'.format(f)

for f in get_files('print/ui', '.ui'):
  xml += '    <file>print/ui/{0}</file>\n'.format(f)

xml += '\n'

xml += '''
    <file compressed="true">icons/hicolor.index.theme</file>
'''

for s in ['16x16', '32x32', '64x64', 'scalable']:
  for c in ['actions', 'categories', 'emblems', 'emotes', 'devices', 'mimetypes', 'places', 'status']:
    icons_dir = 'icons/{0}/{1}'.format(s,c)
    if os.path.exists(os.path.join(srcdir,icons_dir)):
      for f in get_files(icons_dir, '.png'):
        xml += '    <file>icons/{0}/{1}/{2}</file>\n'.format(s,c,f)
      for f in get_files(icons_dir, '.svg'):
        xml += '    <file preprocess=\'xml-stripblanks\'>icons/{0}/{1}/{2}</file>\n'.format(s,c,f)

for f in get_files('inspector', '.ui'):
  xml += '    <file preprocess=\'xml-stripblanks\'>inspector/{0}</file>\n'.format(f)

xml += '''
    <file>inspector/inspector.css</file>
    <file>emoji/en.data</file>
    <file alias="compose/sequences">compose/sequences-{0}-endian</file>
    <file>compose/chars</file>
  </gresource>
</gresources>'''.format(endian)

if len(sys.argv) > 3:
  outfile = sys.argv[3]
  tmpfile = outfile + '~'
  with open(tmpfile, 'w') as f:
    f.write(xml)
  replace_if_changed(tmpfile, outfile)
else:
  print(xml)



#!/usr/bin/env -S GI_TYPELIB_PATH=${PWD}/build/gtk:${GI_TYPELIB_PATH} LD_PRELOAD=${LD_PRELOAD}:${PWD}/build/gtk/libgtk-4.so python3

import gi

gi.require_version('Gdk', '4.0')
gi.require_version('Gtk', '4.0')

from gi.repository import Gdk
from gi.repository import Gtk
from gi.repository import Graphene


class DemoWidget(Gtk.Widget):

    __gtype_name__ = "DemoWidget"

    def __init__(self):
        super().__init__()

    def do_measure(self, orientation, for_size: int):
        # We need some space to draw
        return 100, 200, -1, -1

    def do_snapshot(self, snapshot):
        # Draw four color squares
        color = Gdk.RGBA()
        rect = Graphene.Rect.alloc()

        width = self.get_width() / 2
        height = self.get_height() / 2

        Gdk.RGBA.parse(color, "red")
        rect.init(0, 0, width, height)
        snapshot.append_color(color, rect)

        Gdk.RGBA.parse(color, "green")
        rect.init(width, 0, width, height)
        snapshot.append_color(color, rect)

        Gdk.RGBA.parse(color, "yellow")
        rect.init(0, height, width, height)
        snapshot.append_color(color, rect)

        Gdk.RGBA.parse(color, "blue")
        rect.init(width, height, width, height)
        snapshot.append_color(color, rect)

def on_activate(app):
    # Create a new window
    win = Gtk.ApplicationWindow(application=app)
    win.set_title("Squares")
    icon = DemoWidget()
    win.set_child(icon)
    win.present()

# Create a new application
app = Gtk.Application(application_id='org.gtk.exampleapp')
app.connect('activate', on_activate)

# Run the application
app.run(None)



#!/usr/bin/env python3
#
# SPDX-FileCopyrightText: 2022 Collabora Inc.
#                         2023 Emmanuele Bassi
#
# SPDX-License-Identifier: LGPL-2.1-or-later
#
# Original author: Xavier Claessens <xclaesse@gmail.com>

import argparse
import textwrap
from pathlib import Path


# Disable line length warnings as wrapping the C code templates would be hard
# flake8: noqa: E501


def gen_versions_macros(args, current_major_version, current_minor_version, current_micro_version):
    with args.out_path.open("w", encoding="utf-8") as ofile, args.in_path.open(
        "r", encoding="utf-8"
    ) as ifile:
        for line in ifile.readlines():
            if "@GDK_VERSIONS@" in line:
                ofile.write(
                    textwrap.dedent(
                        f"""\
                    /**
                     * GDK_MAJOR_VERSION:
                     *
                     * The major version component of the library's version, e.g. "1" for "1.2.3".
                     */
                    #define GDK_MAJOR_VERSION ({current_major_version})

                    /**
                     * GDK_MINOR_VERSION:
                     *
                     * The minor version component of the library's version, e.g. "2" for "1.2.3".
                     */
                    #define GDK_MINOR_VERSION ({current_minor_version})

                    /**
                     * GDK_MICRO_VERSION:
                     *
                     * The micro version component of the library's version, e.g. "3" for "1.2.3".
                     */
                    #define GDK_MICRO_VERSION ({current_micro_version})
                        """
                        )
                    )
                for minor in range(0, current_minor_version + 2, 2):
                    ofile.write(
                        textwrap.dedent(
                            f"""\
                        /**
                         * GDK_VERSION_{current_major_version}_{minor}:
                         *
                         * A macro that evaluates to the {current_major_version}.{minor} version of GTK, in a format
                         * that can be used by the C pre-processor.
                         *
                         * Since: {current_major_version}.{minor}
                         */
                        #define GDK_VERSION_{current_major_version}_{minor} (G_ENCODE_VERSION ({current_major_version}, {minor}))
                        """
                        )
                    )
            else:
                ofile.write(line)


def gen_visibility_macros(args, current_major_version, current_minor_version, current_micro_version):
    """
    Generates a set of macros for each minor stable version of GTK

    - GDK_DEPRECATED
    - GDK_DEPRECATED_IN_…
    - GDK_DEPRECATED_MACRO_IN_…
    - GDK_DEPRECATED_ENUMERATOR_IN_…
    - GDK_DEPRECATED_TYPE_IN_…

    - GDK_AVAILABLE_IN_ALL
    - GDK_AVAILABLE_IN_…
    - GDK_AVAILABLE_STATIC_INLINE_IN_…
    - GDK_AVAILABLE_MACRO_IN_…
    - GDK_AVAILABLE_ENUMERATOR_IN_…
    - GDK_AVAILABLE_TYPE_IN_…

    - GDK_UNAVAILABLE(maj,min)
    - GDK_UNAVAILABLE_STATIC_INLINE(maj,min)
    """

    ns = args.namespace
    with args.out_path.open("w", encoding="utf-8") as f:
        f.write(
            textwrap.dedent(
                f"""\
            #pragma once

            #if (defined(_WIN32) || defined(__CYGWIN__)) && !defined({ns}_STATIC_COMPILATION)
            #  define _{ns}_EXPORT __declspec(dllexport)
            #  define _{ns}_IMPORT __declspec(dllimport)
            #elif __GNUC__ >= 4
            #  define _{ns}_EXPORT __attribute__((visibility("default")))
            #  define _{ns}_IMPORT
            #else
            #  define _{ns}_EXPORT
            #  define _{ns}_IMPORT
            #endif
            #ifdef GTK_COMPILATION
            #  define _{ns}_API _{ns}_EXPORT
            #else
            #  define _{ns}_API _{ns}_IMPORT
            #endif

            #define _{ns}_EXTERN _{ns}_API extern

            #define {ns}_VAR _{ns}_EXTERN
            #define {ns}_AVAILABLE_IN_ALL _{ns}_EXTERN

            #ifdef GDK_DISABLE_DEPRECATION_WARNINGS
            #define {ns}_DEPRECATED _{ns}_EXTERN
            #define {ns}_DEPRECATED_FOR(f) _{ns}_EXTERN
            #define {ns}_UNAVAILABLE(maj,min) _{ns}_EXTERN
            #define {ns}_UNAVAILABLE_STATIC_INLINE(maj,min)
            #else
            #define {ns}_DEPRECATED G_DEPRECATED _{ns}_EXTERN
            #define {ns}_DEPRECATED_FOR(f) G_DEPRECATED_FOR(f) _{ns}_EXTERN
            #define {ns}_UNAVAILABLE(maj,min) G_UNAVAILABLE(maj,min) _{ns}_EXTERN
            #define {ns}_UNAVAILABLE_STATIC_INLINE(maj,min) G_UNAVAILABLE(maj,min)
            #endif
            """
            )
        )
        for minor in range(0, current_minor_version + 2, 2):
            f.write(
                textwrap.dedent(
                    f"""
                #if GDK_VERSION_MIN_REQUIRED >= GDK_VERSION_4_{minor}
                #define {ns}_DEPRECATED_IN_{current_major_version}_{minor} {ns}_DEPRECATED
                #define {ns}_DEPRECATED_IN_{current_major_version}_{minor}_FOR(f) {ns}_DEPRECATED_FOR (f)
                #define {ns}_DEPRECATED_MACRO_IN_{current_major_version}_{minor} GDK_DEPRECATED_MACRO
                #define {ns}_DEPRECATED_MACRO_IN_{current_major_version}_{minor}_FOR(f) GDK_DEPRECATED_MACRO_FOR (f)
                #define {ns}_DEPRECATED_ENUMERATOR_IN_{current_major_version}_{minor} GDK_DEPRECATED_ENUMERATOR
                #define {ns}_DEPRECATED_ENUMERATOR_IN_{current_major_version}_{minor}_FOR(f) GDK_DEPRECATED_ENUMERATOR_FOR (f)
                #define {ns}_DEPRECATED_TYPE_IN_{current_major_version}_{minor} GDK_DEPRECATED_TYPE
                #define {ns}_DEPRECATED_TYPE_IN_{current_major_version}_{minor}_FOR(f) GDK_DEPRECATED_TYPE_FOR (f)
                #else
                #define {ns}_DEPRECATED_IN_{current_major_version}_{minor} _{ns}_EXTERN
                #define {ns}_DEPRECATED_IN_{current_major_version}_{minor}_FOR(f) _{ns}_EXTERN
                #define {ns}_DEPRECATED_MACRO_IN_{current_major_version}_{minor}
                #define {ns}_DEPRECATED_MACRO_IN_{current_major_version}_{minor}_FOR(f)
                #define {ns}_DEPRECATED_ENUMERATOR_IN_{current_major_version}_{minor}
                #define {ns}_DEPRECATED_ENUMERATOR_IN_{current_major_version}_{minor}_FOR(f)
                #define {ns}_DEPRECATED_TYPE_IN_{current_major_version}_{minor}
                #define {ns}_DEPRECATED_TYPE_IN_{current_major_version}_{minor}_FOR(f)
                #endif

                #if GDK_VERSION_MAX_ALLOWED < GDK_VERSION_{current_major_version}_{minor}
                #define {ns}_AVAILABLE_IN_{current_major_version}_{minor} {ns}_UNAVAILABLE ({current_major_version}, {minor})
                #define {ns}_AVAILABLE_STATIC_INLINE_IN_{current_major_version}_{minor} GDK_UNAVAILABLE_STATIC_INLINE ({current_major_version}, {minor})
                #define {ns}_AVAILABLE_MACRO_IN_{current_major_version}_{minor} GDK_UNAVAILABLE_MACRO ({current_major_version}, {minor})
                #define {ns}_AVAILABLE_ENUMERATOR_IN_{current_major_version}_{minor} GDK_UNAVAILABLE_ENUMERATOR ({current_major_version}, {minor})
                #define {ns}_AVAILABLE_TYPE_IN_{current_major_version}_{minor} GDK_UNAVAILABLE_TYPE ({current_major_version}, {minor})
                #else
                #define {ns}_AVAILABLE_IN_{current_major_version}_{minor} _{ns}_EXTERN
                #define {ns}_AVAILABLE_STATIC_INLINE_IN_{current_major_version}_{minor}
                #define {ns}_AVAILABLE_MACRO_IN_{current_major_version}_{minor}
                #define {ns}_AVAILABLE_ENUMERATOR_IN_{current_major_version}_{minor}
                #define {ns}_AVAILABLE_TYPE_IN_{current_major_version}_{minor}
                #endif
                """
                )
            )


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("gtk_version", help="Current GLib version")
    subparsers = parser.add_subparsers()

    versions_parser = subparsers.add_parser(
        "versions-macros", help="Generate versions macros"
    )
    versions_parser.add_argument("in_path", help="input file", type=Path)
    versions_parser.add_argument("out_path", help="output file", type=Path)
    versions_parser.set_defaults(func=gen_versions_macros)

    visibility_parser = subparsers.add_parser(
        "visibility-macros", help="Generate visibility macros"
    )
    visibility_parser.add_argument("namespace", help="Macro namespace")
    visibility_parser.add_argument("out_path", help="output file", type=Path)
    visibility_parser.set_defaults(func=gen_visibility_macros)

    args = parser.parse_args()
    version = [int(i) for i in args.gtk_version.split(".")]
    args.func(args, version[0], version[1], version[2])


if __name__ == "__main__":
    main()
